
import { SGraphItem } from "@saga-web/graph/lib";
import { SPolygonItem } from "@/components/mapClass/SPolygonItem";
import { Legend } from '../types/Legend';
import { SPainter, SColor, SFont, SPoint } from "@saga-web/draw";
import { STextItem } from '@saga-web/graph/lib';
import { hexify } from "@/components/mapClass/until"
import { SItemStatus, ItemOrder } from '@saga-web/big/lib';
/**
 * 图例节点Item(区域类型)
 *
 * * @author  张宇(taohuzy@163.com)
 */
export class SZoneLegendItem extends SPolygonItem {
    /** 图例节点对象数据 */
    data: Legend;

    /** text item   */
    textItem: STextItem = new STextItem(this);

    get text(): string {
        return this.textItem.text;
    }
    set text(v: string) {
        this.textItem.text = v;
        this.update();
    }
    get color(): string {
        return this.textItem.color;
    }
    set color(v: string) {
        this.textItem.color = v
    }
    get font(): SFont {
        return this.textItem.font
    }
    set font(v: SFont) {
        this.textItem.font = v
    }

    get status(): SItemStatus {
        return this._status;
    }
    // 编辑当前状态
    set status(value: SItemStatus) {
        this._status = value;
        // 如果状态为show则需清栈
        if (value == SItemStatus.Normal) {
            // 计算文本位置
            let x: number = this.getPointList.reduce((pre, cur, index, arr) => {
                                return pre + (cur.x / arr.length)
                            }, 0),
                y: number = this.getPointList.reduce((pre, cur, index, arr) => {
                                return pre + (cur.y / arr.length)
                            }, 0);
            this.textItem.moveTo(x, y);
            this.showText = true;
            if (this.undoStack) {
                this.undoStack.clear();
            }
        } else if (value == SItemStatus.Edit) {
            this.showText = true;
        } else if (value == SItemStatus.Create) {
            this.showText = false;
        }
        this.update();
    };

    /** 是否显示文字  */
    _showText: boolean = true;
    get showText(): boolean {
        return this._showText;
    }
    set showText(v: boolean) {
        if (v === this._showText) {
            return
        }
        this._showText = v;
        if (v) {
            this.textItem.show();
        } else {
            this.textItem.hide();
        }
    }

    /**
     * 构造函数
     *
     * @param parent    指向父对象
     * @param data      图例节点对象数据
     */
    constructor(parent: SGraphItem | null, data: Legend) {
        super(parent);
        this.data = data;
        this.id = data.ID;
        this.name = data.Name;
        this.font = new SFont("sans-serif", 2);
        if (data) {
            this.setPointList = [];
            let setPointList: SPoint[];
            if (data.OutLine) {
                if (data.OutLine[0][0] instanceof SPoint) {
                    this.setPointList = data.OutLine[0];
                } else {
                    setPointList = data.OutLine[0].map(i => {
                        return (new SPoint(i.X, i.Y))
                    })
                    this.setPointList = setPointList;
                }
            }
            if (data.Properties.Text) {
                this.text = data.Properties.Text;
            }
            // 设置线宽
            if (data.Properties.LineWidth) {
                this.lineWidth = data.Properties.LineWidth
            }
            if (data.Properties.strokeColor) {
                this.strokeColor = data.Properties.strokeColor.includes('#') ? new SColor(data.Properties.strokeColor) : new SColor(hexify(data.Properties.strokeColor));
            }
            if (data.Properties.fillColor) {
                this.fillColor = data.Properties.fillColor.includes('#') ? new SColor(data.Properties.fillColor) : new SColor(hexify(data.Properties.fillColor))
            }
            //    if( data.Properties.LineDash){
            //     this.LineDash =this._legend.Properties.LineDash
            //    }
            console.log(this);
        }
    }
    toData(): any {
        this.data.Properties.fillColor = this.fillColor.value;
        this.data.Properties.strokeColor = this.strokeColor.value;
        this.data.Properties.lineWidth = this.lineWidth;
        this.data.OutLine = [this.getPointList];
        return this.data;
    }

} // Class SZoneLegendItem
